from baraqdalib import Generator
import csv
import random
from typing import List, Dict
import pkgutil

class Addresses:
    """Class Addresses for generating polish fake address but with true distribution"""
    def __init__(self):
        self.address_generator = Generator()

        self.streets_file_name: str = 'streets.csv'
        self.cities_file_name: str = 'cities.csv'
        self.cities_pops_data: str = 'cities_pops'
        self.cities_pops_file_name: str = 'cities_pops.csv'

        self._postal_code: List[List[str, str]] = list(list())
        self._streets: List[List[str, str]] = list(list())
        self._cities: List[List[str, str]] = list(list())

        self.sep: str = '\t'

        self._set_streets()
        self._set_cities()
        self._set_codes()

    def _set_codes(self):   # Setting self._postal_code variable with list from 'postal_codes.csv'
        """Get postal codes from file postal_codes.csv and save in self._postal_code list

        Parameters : None

        Returns: None
        """
        codesFile = pkgutil.get_data(__package__, 'addressData/postal_codes.csv')
        codes = csv.reader(codesFile.decode('utf-8-sig').splitlines(), delimiter=';')
        for code in codes:
            self._postal_code.append([code[2], code[0]])

    def _set_cities(self):   # Setting self.cities variable with list from self.cities_file_name
        """Get cities names  from file cities.csv and save in self._cities list

        Parameters : None

        Returns: None
        """
        citiesFile = pkgutil.get_data(__package__, 'addressData/cities.csv')
        cities = csv.reader(citiesFile.decode('utf-8-sig').splitlines(), delimiter=';')
        for sym in cities:
            self._cities.append(sym)

    def _set_streets(self):  # Setting self.streets variable with list from self.cities_file_name
        """Get street names from file streets.csv and save in self._streets list

        Parameters : None

        Returns: None
        """
        streetsFile = pkgutil.get_data(__package__, 'addressData/streets.csv')
        streets_csv = csv.reader(streetsFile.decode('utf-8-sig').splitlines())
        for row in streets_csv:
            self._streets.append([row[4], (row[6] + ' ' + row[8] + ' ' + row[7]).replace('  ', ' ')])

    def generate(self, counter: int = 1, lang: str = 'PL'):   # Generating from Generator class
        """ Generating address with city, street, street number,  and postal code

        Parameters:
        counter (int): Default is 1. You can specify how much addreses you want to generate
        lang (str): Default is 'PL'. By this argument you can choose from wchich country addresses should be generated.

        Returns:
        Dict(Dict()): returning generated addresses in nested dictionary for easy access

        """
        generated_cities = self.address_generator.generate(lang, self.cities_pops_data, counter, self.sep)
        address: Dict[Dict[str, str]] = dict(dict())
        for city, address_id in zip(generated_cities, range(len(generated_cities))):
            sym = self.get_sym_city(city)
            street = self.get_streets(int(sym))
            postal_code = self.get_code(str(city))
            address.update({str(address_id): {'street': street, 'city': city, 'postal_code': postal_code}})
        return address

    def get_code(self, city: str):  # Searching for city in postal_codes returning it's value
        """Search and return postal code for a given city

        Parameters:
        city (str): Name of the city

        Returns:
        str: Returns postal code
        """
        codes = list()
        for code in self._postal_code:
            if code[0] == city:
                codes.append(code[1])
        if codes:
            return random.choice(codes)
        return 'No city in file'

    def get_sym_city(self, city: str):    # Searching for city in cities and returning sym
        """Searching for city in cities and returning sym (indentificator)

        Parameters:
        city (str): name of city for searching

        Returns:
        str : return sym for searching city
        """
        for sym in self._cities:
            if sym[1] == city:
                return sym[0]
        return 'No city in file'

    def get_streets(self, city_sym: int):    # Returning streets for city sym
        """ Generate random street for given city sym (identificator) that is in this city.

        Parameters:
        city_sym (int): number of city syn

        Returns:
        str: returns random street
        """
        if city_sym != 'No city in file':   # If city_sym isn't set as a 'No city in file' generating address
            streets = []
            streets_dump = []
            for row in self._streets:   # Reading streets
                if row[0] == city_sym:      # Checking if city is in list
                    streets.append(row[1])
                elif row[0] != 'SYM' and row[0] != city_sym:    # Checking if city isn't in list and isn't header 'SYM'
                    if 287400 % int(row[0]):        # Decimation of list of streets from which we generate
                        streets_dump.append(row[1])
            if not streets:
                return random.choice(streets_dump) + ' ' + str((round(random.lognormvariate(1.6, 2)) + 1) % 200)    # Generating address if city isn't in file
            return random.choice(streets) + ' ' + str((round(random.lognormvariate(1.6, 2)) + 1) % 200)     # Generating address if city is in file
        else:
            return city_sym       # If city_sym is set as a 'No city in file' returning this string



import csv
import random
import os
from typing import List, Dict
import pkgutil


class Generator:
    """Generating data with real distribution given in loaded files. To call functions simply use Generator.generate()."""
    def __init__(self) -> None:
        self._data: Dict[str, Dict[str, Dict[str, list]]] = dict(dict(dict()))  # create empty dict
        self._list_of_files: Dict[str, list] = dict()

        self._list_of_files = {'PL': ['age.csv', 'blood_type.csv', 'cities_pops.txt', 'eyes.csv', 'female_first_name.csv', 'female_second_name.csv', 'female_surname.csv',
                      'hair.csv', 'male_first_name.csv', 'male_second_name.csv', 'male_surname.csv']}

    def draw(self, lang: str, data_type: str, count: int = 1, sep: str = ' ') -> List[str]:  # return table with weighted draw
        """Drawing a results from given file, returing list with len(list) = counter

        Parameters:
        lang (str): two letters shortcut for country
        data_type (str): name of file from which data should be generated
        count (int): default is 1. How many records should be generated
        sep (str): default is single space. Set separators for columns in files

        Returns:
        list: Returning list of generated data.
        """
        try:
            return random.choices(self._data[lang][data_type]['values'],
                                  weights=self._data[lang][data_type]['weights'],
                                  k=count)
        except KeyError:  # run normal generation when error occurred
            self.generate(lang, data_type, count, sep)

    def search_files(self, path, sep, lang) -> None:
        """Searching for files in given list of filenames

        Parameters:
        sep (str): separator of columns in files
        lang (str): Two letters shortcut for country

        Returns: None
        """
        self._data[lang] = {}
        for file in self._list_of_files[lang]:
            filename = file.split('.')[0]
            self._data[lang][filename] = {}  # create dict with key filename
            self.read_files(os.path.join('data/PL', file), sep, lang, filename)

    def read_files(self, filepath: str, separator: str, lang: str, filename: str) -> None:
        """Reading files into class variable self._data

        Parameters:
        filepath (str): Filepath for a file
        separator (str): Separator of columns in files
        lang (str): Two letters shortcut for country
        filename (str): Filename

        Returns: None
        """
        # read files and store data from them
        files = pkgutil.get_data(__package__, filepath)
        file = csv.reader(files.decode('utf-8').splitlines(), delimiter='\t')
        temp_list = []
        keys = []
        keys = next(file)
        weights_index = keys.index('weights')
        for i in range(len(keys)):  # create lists to store values
            temp_list.append([])
        for row in file:
            if row[weights_index] == 'weights':
                pass
            else:
                temp_tab = row
                temp_tab[weights_index] = int(temp_tab[weights_index])
                for t, item in zip(temp_tab, temp_list):
                    item.append(t)
        for k, v in zip(keys, temp_list):
            self._data[lang][filename][k] = v

    def generate(self, lang: str, data_type: str, counter: int = 1, sep: str = ' ') -> List[str]:
        """Generating data with reading files. This is recommended way of using.

        Parameters:
        lang (str): two letters shortcut for country
        data_type (str): name of file from which data should be generated
        counter (int): default is 1. How many records should be generated
        sep (str): default is single space. Set separators for columns in files

        Returns:
        list: Returning list of generated data.
        """
        # check if key lang exist
        try:
            self._data[lang]
        except KeyError:
            path = os.path.join('baraqdalib', 'data', lang)  # create path to subdirectories
            self.search_files(path, sep, lang)  # search files in directory
            try:  # check if key lang exist after search for directory
                self._data[lang]
            except KeyError:
                print(f'Data not provided for {lang}!')
                return []

        # check if key data_type exist
        try:
            self._data[lang][data_type]
        except KeyError:
            path = os.path.join('baraqdalib', 'data', lang)  # create path to subdirectories
            self.search_files(path, sep, lang)  # search files in directory
            try:  # check if key data_type exist after search and reading files
                self._data[lang][data_type]
            except KeyError:
                print(f'Data not provided for {lang}, {data_type}!')
                return []
        return self.draw(lang, data_type, counter, sep)  # generate weighted dataw

    def access_data(self, lang: str, data_type: str) -> Dict[str, list]:
        """Access data which are readed from files. Helpful when you
        want to make sure that your files are in correct format

        Parameters:
        lang (str): Two letters shortcut for country
        data_type (str): Name of file from which data should be generated

        Returns:
        Dict: Returning dictionary with loaded data in class instance
        """
        try:  # check if keys lang, data_type exists
            self._data[lang][data_type]
        except KeyError:
            print(f'No data for {lang}, {data_type}')
        else:
            return self._data[lang][data_type]
